#!/usr/bin/ruby -w

require 'test/unit'
require 'fwt'

class FWT_test < Test::Unit::TestCase
  def test_hadamard
    assert_equal [1,0,0,0,0,0,0,0].hadamard, [1, 1, 1, 1, 1, 1, 1, 1]
    assert_equal [0,1,0,0,0,0,0,0].hadamard, [1, -1, 1, -1, 1, -1, 1, -1]
    assert_equal [0,0,1,0,0,0,0,0].hadamard, [1, 1, -1, -1, 1, 1, -1, -1]
    assert_equal [0,0,0,1,0,0,0,0].hadamard, [1, -1, -1, 1, 1, -1, -1, 1]
    assert_equal [0,0,0,0,1,0,0,0].hadamard, [1, 1, 1, 1, -1, -1, -1, -1]
    assert_equal [0,0,0,0,0,1,0,0].hadamard, [1, -1, 1, -1, -1, 1, -1, 1]
    assert_equal [0,0,0,0,0,0,1,0].hadamard, [1, 1, -1, -1, -1, -1, 1, 1]
    assert_equal [0,0,0,0,0,0,0,1].hadamard, [1, -1, -1, 1, -1, 1, 1, -1]
    assert_equal [0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1].sequency,
      [1, -1, 1, -1, 1, -1, 1, -1, 1, -1, 1, -1, 1, -1, 1, -1]
    assert_raise RuntimeError do   # non-power-of-2 array
      [0,0,1].hadamard
    end
  end

  def test_sequency
    assert_equal [1,0,0,0,0,0,0,0].sequency, [1, 1, 1, 1, 1, 1, 1, 1]
    assert_equal [0,1,0,0,0,0,0,0].sequency, [1, 1, 1, 1, -1, -1, -1, -1]
    assert_equal [0,0,1,0,0,0,0,0].sequency, [1, 1, -1, -1, -1, -1, 1, 1]
    assert_equal [0,0,0,1,0,0,0,0].sequency, [1, 1, -1, -1, 1, 1, -1, -1]
    assert_equal [0,0,0,0,1,0,0,0].sequency, [1, -1, -1, 1, 1, -1, -1, 1]
    assert_equal [0,0,0,0,0,1,0,0].sequency, [1, -1, -1, 1, -1, 1, 1, -1]
    assert_equal [0,0,0,0,0,0,1,0].sequency, [1, -1, 1, -1, -1, 1, -1, 1]
    assert_equal [0,0,0,0,0,0,0,1].sequency, [1, -1, 1, -1, 1, -1, 1, -1]
    assert_equal [0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0].sequency,
      [1, 1, 1, 1, 1, 1, 1, 1, -1, -1, -1, -1, -1, -1, -1, -1]
    assert_raise RuntimeError do   # non-power-of-2 array
      [0,0,1].sequency
    end
    assert_raise RuntimeError do   # empty array
      [].sequency
    end
  end

  def test_power_of_2
    assert_true 1.power_of_2?
    assert_true 2.power_of_2?
    assert_true 4.power_of_2?
    assert_true 8.power_of_2?
    assert_true 16.power_of_2?
    assert_true (2**32).power_of_2?
    assert_true (2**500).power_of_2?
    assert_false 0.power_of_2?
    assert_false 3.power_of_2?
    assert_false 5.power_of_2?
    assert_false 7.power_of_2?
    assert_false (-1).power_of_2?
    assert_false (-2).power_of_2?
    assert_false (-3).power_of_2?
    assert_false (-4).power_of_2?
    assert_false (2**32 - 1).power_of_2?
    assert_false (2**500 - 1).power_of_2?
  end

end
